package com.chrismweb.box2dforflashgames;


import java.util.ArrayList;
import java.util.Iterator;

import com.badlogic.gdx.ApplicationListener;
import com.badlogic.gdx.Game;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.GL10;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.Texture.TextureFilter;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.g2d.BitmapFont.HAlignment;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;
import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyDef;
import com.badlogic.gdx.physics.box2d.Box2DDebugRenderer;
import com.badlogic.gdx.physics.box2d.CircleShape;
import com.badlogic.gdx.physics.box2d.FixtureDef;
import com.badlogic.gdx.physics.box2d.PolygonShape;
import com.badlogic.gdx.physics.box2d.World;
import com.badlogic.gdx.physics.box2d.BodyDef.BodyType;


/**
 * ...
 * #Chp2.2 - Totem Level Setup
 * @book by Emanuele Feronato
 * http://www.emanueleferonato.com/
 * Get it now @ http://www.packtpub.com/box2d-for-flash-games/book
 */

public class MyGdxGame implements ApplicationListener
{
	//for drawing
	private OrthographicCamera camera;
	private SpriteBatch batch;
	
	//
	Vector3 touchPoint;
	
	

	public static float SCREEN_WIDTH_ORIGINAL=640;
	public static float SCREEN_HEIGHT_ORIGINAL=480;
	
	public static float SCREEN_WIDTH=640;
	public static float SCREEN_HEIGHT=480;
	
	//for testing other screen resolutions quickly in the desktop version
//	public static float SCREEN_WIDTH=1280;
//	public static float SCREEN_HEIGHT=800;
	
	public static float BOX2D_WIDTH_SCALE=1;
	public static float BOX2D_HEIGHT_SCALE=1;
	
	//box2d variables
	private World world;
	private Box2DDebugRenderer debugRenderer;
	public static float PIXELS_PER_METER = 30.0f;
	
	//reset variables
	BitmapFont font;
	Rectangle rect_reset;
	private ShapeRenderer renderer;
	
	

	@Override
	public void create()
	{
		SCREEN_WIDTH= Gdx.graphics.getWidth();
		SCREEN_HEIGHT= Gdx.graphics.getHeight();
		
		//NOTE: with different screen sizes (on android for instance) the world will be setup as different sizes, so the mass of objects won't be the same.
		//you need to create the world the same size, and scale the camera, so that it behaves the same regardless of screen resolution
		BOX2D_WIDTH_SCALE=SCREEN_WIDTH_ORIGINAL/SCREEN_WIDTH;
		BOX2D_HEIGHT_SCALE=SCREEN_HEIGHT_ORIGINAL/SCREEN_HEIGHT;
		
		
		// setup the camera. In Box2D we operate on a
		// meter scale, pixels won't do it. So we use
		// an orthographic camera with a viewport of
		// 48 meters in width and 32 meters in height.
		// We also position the camera so that it
		// looks at (0,16) (that's where the middle of the
		// screen will be located).
		camera = new OrthographicCamera(PixToMeter(SCREEN_WIDTH)*BOX2D_WIDTH_SCALE, PixToMeter(SCREEN_HEIGHT)*BOX2D_HEIGHT_SCALE);
		camera.position.set(PixToMeter(SCREEN_WIDTH)/2*BOX2D_WIDTH_SCALE, PixToMeter(SCREEN_HEIGHT)/2*BOX2D_HEIGHT_SCALE, 0);
				
		batch = new SpriteBatch();
		
		touchPoint = new Vector3(0,0,0);

		Vector2 gravity=new Vector2(0.0f, -9.81f); //we have to flip ALL y values- AS3 uses down int he y duirection as increasing, whereas OpenGL uses UP in the y direction as positive
		Boolean sleep = true;
		world = new World(gravity, sleep);
		debugRenderer = new Box2DDebugRenderer();
		
		Setup();
		
		font = new BitmapFont(Gdx.files.internal("resources/fonts/arial-15.fnt"), false);
		font.setScale(1/BOX2D_HEIGHT_SCALE);
		
		rect_reset = new Rectangle(0, SCREEN_HEIGHT-30/BOX2D_HEIGHT_SCALE, 100/BOX2D_WIDTH_SCALE, 30/BOX2D_HEIGHT_SCALE);
		renderer = new ShapeRenderer();
		
		Gdx.app.log("MyGdxGame", "GAME STARTED");
		
	}
	
	private void ResetWorld()
	{
		Gdx.app.log("MyGdxGame", "Resetting world");
		while(world.getBodyCount()!=0)//for some reason doesn't  delte all bodies the first time....
		{
			Iterator<Body> iterator = world.getBodies();
			while(iterator.hasNext()) 
			{
		         Body body = iterator.next();
	//	         body.destroyFixture(fixture)
	//	         body.destroyFixture(body.getFixtureList().get(0));
		         if(body!=null)
		        	 world.destroyBody(body);
			}
		}
		
		Setup();
	}
	
	public void Setup()
	{
		Brick(275,435,30,30);
		Brick(365,435,30,30);
		Brick(320,405,120,30);
		Brick(320,375,60,30);
		Brick(305,345,90,30);
		Brick(320,300,120,60);
		
		Idol(320, 242);
		
		Floor();
	}
	
	private void Brick(float pX, float pY, float w, float h) 
	{
		
		BodyDef bodyDef = new BodyDef();
		bodyDef.position.set(PixToMeter(pX),PixToMeter(MyGdxGame.SCREEN_HEIGHT_ORIGINAL-pY));
		bodyDef.type = BodyType.DynamicBody;
		PolygonShape polygonShape=new PolygonShape();
		polygonShape.setAsBox(PixToMeter(w/2),PixToMeter(h/2));
		FixtureDef fixtureDef = new FixtureDef();
		fixtureDef.shape=polygonShape;
		fixtureDef.density=2;
		fixtureDef.restitution=0.4f;
		fixtureDef.friction=0.5f;
		Body thebrick=world.createBody(bodyDef);
		thebrick.createFixture(fixtureDef);
	}
	
	private void Idol(float pX, float pY) 
	{
		//create the initial axis oriented box
		BodyDef bodyDef = new BodyDef();
		bodyDef.position.set(PixToMeter(pX), PixToMeter(MyGdxGame.SCREEN_HEIGHT_ORIGINAL-pY));
		bodyDef.type = BodyType.DynamicBody;
		
		PolygonShape polygonShape=new PolygonShape();
		polygonShape.setAsBox(PixToMeter(5), PixToMeter(20));
		
		FixtureDef fixtureDef = new FixtureDef();
		fixtureDef.shape=polygonShape;
		fixtureDef.density=1;
		fixtureDef.restitution=0.4f;
		fixtureDef.friction=0.5f;
		Body theIdol=world.createBody(bodyDef);
		theIdol.createFixture(fixtureDef);
		
		//create the 2 45deg angle boxes:
		float bW=PixToMeter(5);
		float bH=PixToMeter(20);
		Vector2 boxPos = new Vector2(0, PixToMeter(-10)); //trhis is actually positioned relative to the position of the first box! ALSO negative for LIBGDX vs AS3
		
		float boxAngle = (float) (-Math.PI / 4); //first the counter-clockwise box PI=3.14rad=180deg / 4= 45
//		polygonShape.setAsOrientedBox(bW,bH,boxPos,boxAngle);
		polygonShape.setAsBox(bW,bH,boxPos,boxAngle);
		
		fixtureDef.shape=polygonShape;
		theIdol.createFixture(fixtureDef);
		
		boxAngle=(float) (Math.PI/4);
		polygonShape.setAsBox(bW,bH,boxPos,boxAngle);
		fixtureDef.shape=polygonShape;
		theIdol.createFixture(fixtureDef);
		
		//now actually create the last box as a polygon by actually specifying individual vertices! Convex and in clockwise order ONLY
		//FOR JAVA- had to change the y values to opposite sign, and since it puts it in counter clockwise, have to reverse the order
		ArrayList<Vector2> vertices = new ArrayList<Vector2>();
		vertices.add(new Vector2(PixToMeter(0),PixToMeter(10)));
		vertices.add(new Vector2(PixToMeter(15), PixToMeter(25)));
		vertices.add(new Vector2(PixToMeter(0),PixToMeter(40)));
		vertices.add(new Vector2(PixToMeter(-15), PixToMeter(25)));
		
		
		
		Vector2[] vertices_arr = new Vector2[vertices.size()];
		for(int i=0;i<vertices_arr.length;i++)
			vertices_arr[i]=vertices.get(i);
		polygonShape.set(vertices_arr);
		fixtureDef.shape=polygonShape;
		theIdol.createFixture(fixtureDef);
	}
	
	private void Floor() 
	{
		BodyDef bodyDef = new BodyDef();
		bodyDef.position.set(PixToMeter(320),PixToMeter(MyGdxGame.SCREEN_HEIGHT_ORIGINAL- 465));
		PolygonShape polygonShape=new PolygonShape();
		polygonShape.setAsBox(PixToMeter(320),PixToMeter(15));
		FixtureDef fixtureDef = new FixtureDef();
		fixtureDef.shape=polygonShape;
		fixtureDef.restitution=0.4f;
		fixtureDef.friction=0.5f;
		Body theFloor=world.createBody(bodyDef);
		theFloor.createFixture(fixtureDef);
	}
	
	private float PixToMeter(float pixels)
	{
		return pixels / PIXELS_PER_METER;
	}
	private float MeterToPix(float meters)
	{
		return meters * PIXELS_PER_METER;
	}

	@Override
	public void dispose()
	{
		batch.dispose();

	}

	@Override
	public void render()
	{
		world.step(Gdx.app.getGraphics().getDeltaTime(), 3, 3);
		
		Gdx.gl.glClearColor(.2f, .2f, .2f, 1); //0->1 = 0->255
		Gdx.gl.glClear(GL10.GL_COLOR_BUFFER_BIT);
		camera.update();

		
		
		debugRenderer.render(world, camera.combined);
		
		
		renderer.setColor(.05f, .2f, .31f, 1.0f);
		renderer.begin(ShapeType.FilledRectangle);
		renderer.filledRect(rect_reset.x, rect_reset.y, rect_reset.width, rect_reset.height);
		renderer.end();
		
		batch.begin();
		font.drawMultiLine(batch, "Reset", rect_reset.x, rect_reset.y+font.getLineHeight(), rect_reset.width, HAlignment.CENTER);
		batch.end();
		
		Update();
	}
	public void Update()
	{
		// translate the mouse coordinates to world coordinates
		if (Gdx.input.justTouched()) 
		{
			camera.unproject(touchPoint.set(Gdx.input.getX(), Gdx.input.getY(), 0));
			Vector2 touchPointPixels =new Vector2(MeterToPix(touchPoint.x)/BOX2D_WIDTH_SCALE, MeterToPix(touchPoint.y)/BOX2D_HEIGHT_SCALE);
			
//			Gdx.app.log("MyGdxGame", "GAME touchPointPixels: "+touchPointPixels);
			if(rect_reset.contains(touchPointPixels.x, touchPointPixels.y))
				ResetWorld();
		}

	}

	@Override
	public void resize(int width, int height)
	{
	}

	@Override
	public void pause()
	{
	}

	@Override
	public void resume()
	{
	}
}
